unit udmdemo;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, SQLite3Conn, SQLDBLib, SQLDB, Forms;

const
  cSQLiteDllName              ='sqlite3.dll';
  cSQLStandardImportFileName  ='standard.sql';
  cSQLCustomImportFileName    ='demo.sql';
  cSQLDatabaseFile            ='demo.db';
  cUNDEF                      ='undefined';
  cDelimiter                  ='@';

type

  { Tdmdemo }

  Tdmdemo = class(TDataModule)
    SQLConnector1: TSQLConnector;
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLScript1: TSQLScript;
    SQLTransaction1: TSQLTransaction;
    procedure DataModuleCreate(Sender: TObject);
  private
    function GuessFile(const sFileName: string; const sWhereToSearch: string;
      var sSearched: string): string;      //tries to find a file in application dirextory and other directories
    function GuessLibraryLocation: string; //versucht die SQLite-DLL in Standardverzeichnissen zu suchen und gibt den Pfad+Dateinamen zurück
    function GuessSQLImportFile:string;     //versucht in standardverzeichnissen das Datenfile für FileDatenbanken (SQLITE etc) zu finden
  public
    procedure OpenConnection;
    procedure CloseConnection;
  end;

var
  dmdemo: Tdmdemo;

implementation

uses
  dialogs;
{$R *.lfm}

{ Tdmdemo }

procedure Tdmdemo.DataModuleCreate(Sender: TObject);
var
  sLibrary:string;
  sImportFile:string;
  sSQLfile:string;
  sSearched:string;
begin

  //in case SQLDBLibraryLoader1 or SQLConnector1 were active in design mode close them
  CloseConnection;
  SQLDBLibraryLoader1.Enabled:=false;            //triggers an internal UnloadLibrary

  sLibrary :='';
  sLibrary := GuessLibraryLocation;              //Search for sqlite Library

  if sLibrary = cUNDEF then exit;               //for security reasons in case the exception fails

  //First: load library
  SQLDBLibraryLoader1.LibraryName:=sLibrary;
  SQLDBLibraryLoader1.ConnectionType:='SQLite3'; //redefine sqlite in case something was changed

  // Hier kracht es -- entweder enabled=true, das löst ein Loadlibrary aus
  //oder Loadlibrary das setzt enabled=true -- keins von beiden funktioniert
  // zudem... wieso ist das dann doppelt??

  SQLDBLibraryLoader1.Enabled:=true;            //triggers an internal LoadLibrary
 // SQLDBLibraryLoader1.LoadLibrary;                //loads Libraty ans sets enabled=true

  //Second: Try to find the SQLite Database file
  //        If it does not exist SQLite creates the named database file automatically
  sSearched:='';
  sSQLFile := GuessFile(cSQLDatabaseFile,'data@sql',sSearched);
  if sSQLFile = cUNDEF then begin //create an new one and search again
     showmessage('No database file found - searched in' + LineEnding+ sSearched + LineEnding + ' creating standard database instead');
     sSQLFile:=cSQLDatabaseFile;
  end;

  // only then establish a connection to the database
  SQLConnector1.DatabaseName:=sSQLFile;
  SQLConnector1.ConnectorType:='SQLite3';

  SQLConnector1.Open;

  //Third: Import demo data
  sImportFile := GuessSQLImportFile;
  if sImportFile = cUNDEF then begin //just in case
     showmessage( 'SQL import file not found');
     //exit;
  end
  else begin
    SQLScript1.Script.LoadFromFile(sImportFile);
  end;


end;



function Tdmdemo.GuessLibraryLocation: string;
var
  s: string;
  sSearched:string;
begin
  result:=cUNDEF;
  sSearched:='';

  result:=GuessFile(cSQLiteDllName,'dll@sql',sSearched);

{ Nur für Windows-OS spezielle Suche im System-Verzeichnis ---CAVE bitness

ür Windows gibt es eine Unit WinDirs (in fpc/source/rtl/win) mit Konstanten
CSIDL_PROGRAMS, CSIDL_PERSONAL usw, und den Funktionen GetWindowsSpecialDir(),
GetWindowsSpecialDirUnicode(), die damit aufgerufen werden
können und das entsprechende Spezial-Verzeichnis zurückgeben.

und auch für andere  OS:
https://www.lazarusforum.de/viewtopic.php?p=103860&sid=8adc0c5cdb6ef3589f446620f3d8a2a4#p103860


s :=  Application.Location + 'dll'+ DirectorySeparator + 'standard'+ DirectorySeparator
  + cSQLiteDllName;
if FileExists(s) then
begin
 result := s;
 exit;
end;
sSearched:=sSearched+s+LineEnding;
}




  if result=cUNDEF  then
  raise Exception.Create( LineEnding + 'udmdemo.Tdmdemo.GuessLibraryLocation: '
                          + LineEnding+LineEnding
                          + 'tried to find Library ' + cSQLiteDllName + ' in '
                          + LineEnding + LineEnding
                          + sSearched
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + cSQLiteDllName +' in one of the searchpaths');

end;

function Tdmdemo.GuessSQLImportFile: string;
var
  s:string;
  sSearchedin:string;
begin
  result:=cUNDEF;
  sSearchedin:='';

  result:=GuessFile(cSQLCustomImportFileName,'dll@data',sSearchedin);
  if result=cUNDEF  then
    result:=GuessFile(cSQLStandardImportFileName,'dll@data',sSearchedin);

  if result=cUNDEF  then
  raise Exception.Create( LineEnding + 'udmdemo.Tdmdemo.GuessSQLImportFile: '
                          + LineEnding+LineEnding
                          + 'tried to find SQL import File '
                          + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName + ' in '
                          + LineEnding + LineEnding
                          + sSearchedin
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName +' in one of the searchpaths');
end;

procedure Tdmdemo.OpenConnection;
begin
  if not SQLConnector1.Connected then SQLConnector1.Open;
end;

procedure Tdmdemo.CloseConnection;
begin
  if SQLConnector1.Connected then SQLConnector1.Close;
end;

function Tdmdemo.GuessFile(const sFileName: string; const sWhereToSearch: string; var sSearched:string ): string;
var
  s:string;
  sl:TStringlist;
  i: integer;
begin
  result:=cUNDEF;

  s :=  Application.Location + sFileName;
  if FileExists(s) then
  begin
   result := s;
   exit;
  end;
  sSearched:=sSearched + s + LineEnding;

  sl := TStringList.Create;
  try
    sl.Delimiter:=cDelimiter;
    sl.DelimitedText := sWhereToSearch;
    for i := 0 to sl.Count - 1 do begin
      s :=  Application.Location + sl[i]+ DirectorySeparator + sFileName;
      if FileExists(s) then
      begin
       result := s;
       break;
      end;
      sSearched:=sSearched + s + LineEnding;
    end;
  finally
    sl.Free;
  end;

end;

end.

